@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* exec.c: command execution */
-/* (C) 2007-2019 magicant */
+/* (C) 2007-2020 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -312,7 +312,7 @@ void exec_pipelines(const pipeline_T *p, bool finally_exit)
 	suppresserrreturn |= suppress;
 
 	bool self = finally_exit && !doing_job_control_now
-	    && !p->next && !p->pl_neg && !any_trap_set && !shopt_pipefail;
+	    && !p->next && !p->pl_neg && !any_trap_set;
 	exec_commands(p->pl_commands, self ? E_SELF : E_NORMAL);
 	if (p->pl_neg) {
 	    if (laststatus == Exit_SUCCESS)
@@ -351,9 +351,7 @@ void exec_pipelines_async(const pipeline_T *p)
 	job->j_status = JS_RUNNING;
 	job->j_statuschanged = true;
 	job->j_legacy = false;
-#ifndef NDEBUG
-	job->j_beingwaitedfor = false;
-#endif
+	job->j_nonotify = false;
 	job->j_pcount = 1;
 
 	set_active_job(job);
@@ -593,6 +591,9 @@ void exec_commands(command_T *c, exec_T type)
 	count++;
     assert(count > 0);
 
+    if (type == E_SELF && shopt_pipefail && count > 1)
+	type = E_NORMAL;
+
     job = xmallocs(sizeof *job, count, sizeof *job->j_procs);
     ps = job->j_procs;
 
@@ -633,9 +634,7 @@ void exec_commands(command_T *c, exec_T type)
 	job->j_status = JS_RUNNING;
 	job->j_statuschanged = true;
 	job->j_legacy = false;
-#ifndef NDEBUG
-	job->j_beingwaitedfor = false;
-#endif
+	job->j_nonotify = false;
 	job->j_pcount = count;
 	set_active_job(job);
 	if (type == E_NORMAL) {
@@ -797,10 +796,8 @@ pid_t exec_process(
     char *argv0 = NULL;
     pid_t cpid = 0;
 
-    current_lineno = c->c_lineno;
+    update_lineno(c->c_lineno);
 
-    /* fork first if `type' is E_ASYNC, the command type is subshell,
-     * or there is a pipe. */
     finally_exit = (type == E_SELF);
     if (finally_exit) {
 	if (c->c_type == CT_SUBSHELL)
@@ -808,6 +805,8 @@ pid_t exec_process(
 	     * subshell directly in this process. */
 	    become_child(false);
     } else {
+	/* fork first if `type' is E_ASYNC, the command type is subshell,
+	 * or there is a pipe. */
 	if (type == E_ASYNC || c->c_type == CT_SUBSHELL
 		|| pi->pi_fromprevfd >= 0 || pi->pi_tonextfds[PIPE_OUT] >= 0) {
 	    sigtype_T sigtype = (type == E_ASYNC) ? t_quitint : 0;
@@ -1238,7 +1237,22 @@ void exec_simple_command(
 	laststatus = Exit_NOTFOUND;
 	break;
     case CT_EXTERNALPROGRAM:
-	assert(finally_exit);
+	if (!finally_exit) {
+	    pid_t cpid = fork_and_reset(0, true, t_leave);
+	    if (cpid < 0) {
+		laststatus = Exit_NOEXEC;
+		break;
+	    } else if (cpid > 0) {
+		wchar_t **namep = wait_for_child(
+			cpid,
+			doing_job_control_now ? cpid : 0,
+			doing_job_control_now);
+		if (namep != NULL)
+		    *namep = joinwcsarray(argv, L" ");
+		break;
+	    }
+	    finally_exit = true;
+	}
 	exec_external_program(ci->ci_path, argc, argv0, argv, environ);
 	break;
     case CT_SPECIALBUILTIN:
@@ -2292,7 +2306,6 @@ int command_builtin_execute(int argc, void **argv, enum srchcmdtype_T type)
 {
     char *argv0 = malloc_wcstombs(argv[0]);
     commandinfo_T ci;
-    bool finally_exit = false;
 
     if (argv0 == NULL) {
 	xerror(EILSEQ, NULL);
@@ -2300,24 +2313,7 @@ int command_builtin_execute(int argc, void **argv, enum srchcmdtype_T type)
     }
 
     search_command(argv0, argv[0], &ci, type);
-    if (ci.type == CT_EXTERNALPROGRAM) {
-	pid_t cpid = fork_and_reset(0, true, t_leave);
-	if (cpid < 0) {
-	    free(argv0);
-	    return Exit_NOEXEC;
-	} else if (cpid > 0) {
-	    wchar_t **namep = wait_for_child(
-		    cpid,
-		    doing_job_control_now ? cpid : 0,
-		    doing_job_control_now);
-	    if (namep != NULL)
-		*namep = joinwcsarray(argv, L" ");
-	    free(argv0);
-	    return laststatus;
-	}
-	finally_exit = true;
-    }
-    exec_simple_command(&ci, argc, argv0, argv, finally_exit);
+    exec_simple_command(&ci, argc, argv0, argv, false);
     free(argv0);
     return laststatus;
 }
