@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* editing.h: main editing module */
-/* (C) 2007-2010 magicant */
+/* (C) 2007-2017 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,15 +25,15 @@
 
 
 extern xwcsbuf_T le_main_buffer;
-extern size_t le_main_index;
+extern size_t le_main_length, le_main_index;
 
-enum le_search_direction { FORWARD, BACKWARD, };
+enum le_search_direction_T { FORWARD, BACKWARD, };
 /* FORWARD:  find the oldest candidate from the ones newer than the current
  * BACKWARD: find the newest candidate from the ones older than the current */
-enum le_search_type { SEARCH_PREFIX, SEARCH_VI, SEARCH_EMACS, };
+enum le_search_type_T { SEARCH_PREFIX, SEARCH_VI, SEARCH_EMACS, };
 
-extern enum le_search_direction le_search_direction;
-extern enum le_search_type le_search_type;
+extern enum le_search_direction_T le_search_direction;
+extern enum le_search_type_T le_search_type;
 extern xwcsbuf_T le_search_buffer;
 extern const struct histlink_T *le_search_result;
 
@@ -65,6 +65,7 @@ extern le_command_func_T
     cmd_eof_if_empty, /*C*/
     cmd_eof_or_delete, /*C*/
     cmd_accept_with_hash, /*C*/
+    cmd_accept_prediction, /*C*/
     cmd_setmode_viinsert, /*C*/
     cmd_setmode_vicommand, /*C*/
     cmd_setmode_emacs, /*C*/
@@ -149,6 +150,9 @@ extern le_command_func_T
     cmd_complete_list, /*C*/
     cmd_complete_all, /*C*/
     cmd_complete_max, /*C*/
+    cmd_complete_max_then_list, /*C*/
+    cmd_complete_max_then_next_candidate, /*C*/
+    cmd_complete_max_then_prev_candidate, /*C*/
     cmd_clear_candidates; /*C*/
 
 /* vi-mode specific commands */
