@@ -1,5 +1,5 @@
 # run-test.sh: runs a set of test cases
-# (C) 2016 magicant
+# (C) 2016-2019 magicant
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,14 +28,6 @@
 set -Ceu
 umask u+rwx
 
-# require yash for alias support and extended exec & ulimit built-ins
-if ! [ "${YASH_VERSION-}" ]; then
-    eprintf '%s: must be run with yash\n' "$0"
-    exit 64 # sysexits.h EX_USAGE
-fi
-
-command -b ulimit -c 0 2>/dev/null || :
-
 ##### Some utility functions and aliases
 
 eprintf() {
@@ -53,7 +45,15 @@ esac
 
 ##### Script startup
 
-exec <&- 3>&- 4>&- 5>&-
+# require yash for alias support and ulimit built-ins
+if ! [ "${YASH_VERSION-}" ]; then
+    eprintf '%s: must be run with yash\n' "$0"
+    exit 64 # sysexits.h EX_USAGE
+fi
+
+command -b ulimit -c 0 2>/dev/null || :
+
+exec </dev/null 3>&- 4>&- 5>&-
 
 # ensure correctness of $PWD
 cd -L .
@@ -156,14 +156,18 @@ $1"
 # If the "use_valgrind" variable is true, Valgrind is used to run the testee,
 # in which case the testee will ignore argv[0].
 testee() (
+    if [ "${posix:+set}" = set ]; then
+	testee="$testee_sh"
+	export TESTEE="$testee"
+    fi
     if ! "$use_valgrind"; then
-	exec ${posix:+-a sh} "$testee" "$@"
+	exec "$testee" "$@"
     else
 	test -r "$abs_suppressions" || abs_suppressions=
 	exec valgrind --leak-check=full --vgdb=no --log-fd=17 \
 	    ${abs_suppressions:+--suppressions="$abs_suppressions"} \
 	    --gen-suppressions=all \
-	    "$testee" ${posix:+-o posix} "$@" \
+	    "$testee" "$@" \
 	    17>>"${valgrind_file-0.valgrind}"
     fi
 )
@@ -375,6 +379,8 @@ abs_test_file="$(absolute "$test_file")"
 abs_suppressions="$(absolute valgrind.supp)"
 cd "$work_dir"
 
+ln -s -- "$testee" sh
+testee_sh="$(absolute sh)"
 export TESTEE="$testee"
 
 . "$abs_test_file"
